#include "webmon.h"
#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netdb.h>
#include <openssl/md5.h>
#include <pthread.h>


void parse_url_to_config_struct(int i, gchar *instr) {
  /* i will rewrite this function as soon as i have the time, its really ugly */
  /* support for user:pass should also be added */
  char tmpstr[1000];
  char *p, *q, *r = NULL;
  
  //printf("parse_url_to_config_struct()\n"); 
  memset(tmpstr,0,sizeof(tmpstr));
  
  strncpy(tmpstr,instr,sizeof(tmpstr)-1); /* we need a \0 in the end */
  if ((strncmp(tmpstr,"http://",7)) != 0) {
    /* since this is a webmonitor we will only support HTTP */
    wi[i].use = 0;
    return;
  }

  wi[i].use = 1;  /* mark as used */

  p = strstr(tmpstr,"//");
  p +=2;
  /* p is pointing on the name of the server (http:// removed) */
  
  if (!(strchr(p,'/'))) {
    /* we do not have a trailing slash on the URL */
    if ((r = strchr(p,':'))) { /* check if we have a specified port */
      /* port is specified */
      r++;
      wi[i].port = atoi(r);
      r--;
      *r = '\0';
    }
    else {
      /* no port specified, 80 is default */
      wi[i].port = 80;
    }
    strncpy(wi[i].server,p,sizeof(wi[i].server)-1);  /* the servername to wi.server */
    strcpy(wi[i].page,"/"); /* we will put a trailing slash on the URL */
    return; 
  }
  else if ((q = strchr(p,'/'))) {
    *q = '\0';
    /* if we get here we have a page or path specified */
    /* first we will check if we have a port */
    if ((r = strchr(p,':'))) {
      r++;
      wi[i].port = atoi(r); /* the number r is pointing on to wi.port */
      r--;
      *r = '\0';
    }
    else {
      wi[i].port = 80; /* if no port was specified, use the default port 80 */
    }
    strncpy(wi[i].server,p,sizeof(wi[i].server)-1); /* the servername to wi.server */
    if (r != NULL) {
      *r = '/';
    }
    *q = '/';
    strncpy(wi[i].page,q,sizeof(wi[i].page)-1);  
    return;
  }
  
}

void set_config_default(void) {
  
  int 	i;
  
  updatetime = 24;               /* update every 24 hours (default) */
  
  for(i=0; i<20; i++) {
    wi[i].use = 0;               /* mark as unused */
    memset(wi[i].server,0,100);  /* null */
    wi[i].port = 80;             /* 80 as default port */
    memset(wi[i].init_md5,0,33); /* null */
    memset(wi[i].new_md5,0,33);  /* null */
  }
}

void *get_page_and_send_data_to_md5_check(void *arg) {
  
  struct sockaddr_in sin;
  struct hostent     *hent;
  int                fd;
  unsigned char      *datap, *dp;
  char               sendstr[2000];
  int                len;
  char               buf[BUFSIZE];
  int		     num;
  void	             *doh = NULL;
  
  num = (int) arg;
  //printf("num = %d\n",num);
  if ((hent = gethostbyname(wi[num].server)) == NULL) {
    /* we wont do anything with errors right now */
    return doh;
  }

  if ((fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1 ) {
    /* we wont do anything with errors right now */
    return doh;
  }
  sin.sin_family = PF_INET;
  sin.sin_port = htons(wi[num].port);
  sin.sin_addr = *((struct in_addr *)hent->h_addr);
  memset(&(sin.sin_zero), '\0', 8);

  if (connect(fd,(struct sockaddr *)&sin, sizeof(struct sockaddr)) == -1) {
    /* we wont do anything with errors right now */
    return doh;
  }
  memset(sendstr,0,2000);
  sprintf(sendstr,"GET http://%s:%d%s HTTP/1.1\nConnection: close\nHost: foo.com\n\n",wi[num].server,wi[num].port,wi[num].page);
  
  datap = dp = (unsigned char *) malloc(MALLOCSIZE);
  /* this is where we will store the data until we check MD5 */
  memset(datap,0,MALLOCSIZE); /* null the malloc() area */

  if ((send(fd,sendstr,strlen(sendstr),0)) == -1) {
    return doh;
  }
  
  while ((len = recv(fd,buf,BUFSIZE,0)) > 0 ) {
    if (len == -1) return doh;
    if (dp-datap+len > MALLOCSIZE-1) break; /* we cant store more data than MALLOCSIZE */
    memcpy(dp,buf,len);
    dp += len;
  }
  /* now we have stored the data in the malloc() area, data begins at datap */
//  printf("%s",datap);
//  printf("sendstr = '%s'\n",sendstr);
  get_md5_to_webinfo_element(num, datap);
  free(datap); /* free the malloc():ed space */
  return doh;
}

void get_md5_to_webinfo_element(int num, const unsigned char *datap) {
  /* this function will get the MD5 from the data starting at datap */
  
  MD5_CTX       ctx;
  unsigned char digest[16];
  char          output[40];
  unsigned char *p;
  int           i;
  
  // printf("In get_md5_to_webinfo_element()\n");
  /* the webserver has sent us an header */
  /* get past the header */
  if (!(p = (unsigned char *) strstr((const char *)datap,"\r\n\r\n"))) {
    /* this should never happen, but just in case we will return */
    //printf("NO CRLFCRLF FOUND!!!\n");
    return;
  }
  p += 4;
  /* the page starts at p */
  MD5_Init(&ctx);
  MD5_Final(digest,&ctx);
  MD5(p,strlen((char *)p),digest);
  for (i=0; i<16 ; i++) {
    sprintf(output+2*i,"%.2x",digest[i]);
  }
  
  /* now we have the MD5sum from the data in output */
  /* the global 'wheretostore' tells us where to store the data */
  
  if (wheretostore == 0) {
    strncpy(wi[num].init_md5,output,32);
    /* we need to mark the config as changed so that gkrellm will save it */
    gkrellm_config_modified();
  }
  else if (wheretostore == 1) {
    strncpy(wi[num].new_md5,output,32);
  }

}

  
void start_thread(void *arg) {
  
  pthread_t tid;
  int ret;
  //pthread_attr_t tattr;
  //pthread_attr_setdetachstate(&tattr,PTHREAD_CREATE_DETACHED);
  ret = pthread_create(&tid, NULL, get_page_and_send_data_to_md5_check, arg);
  //printf("ret = %d\n",ret);
  //ret = pthread_detach(tid);
  return;
}


